----------------------------------------------------------------------------------
-- Company: ETH Zurich
-- Engineer: Pascal Engeler <engeler.pascal@gmail.com>
-- 
-- Create Date:    17:53:31 08/23/2024 
-- Design Name: 
-- Module Name:    Stitch - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
-- The convention is that signals are named after the component that drives them
----------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
use IEEE.NUMERIC_STD.ALL;

library UNISIM;
use UNISIM.Vcomponents.ALL;


entity Stitch is
Port ( 
    CLK100: in STD_LOGIC;
    RX_IN : in  STD_LOGIC;
    OPT_OUT : out  STD_LOGIC_VECTOR(9 downto 0);
    LEDS: out STD_LOGIC_VECTOR(2 downto 0);
    LASER_CLK_1_P: in STD_LOGIC;
    LASER_CLK_1_N: in STD_LOGIC;
    LASER_SIGNAL_1_P: in STD_LOGIC;
    LASER_SIGNAL_1_N: in STD_LOGIC;
    LASER_SIGNAL_2_P: in STD_LOGIC;
    LASER_SIGNAL_2_N: in STD_LOGIC;
    LASER_SIGNAL_3_P: in STD_LOGIC;
    LASER_SIGNAL_3_N: in STD_LOGIC;
    LASER_CLK_2_P: in STD_LOGIC;
    LASER_CLK_2_N: in STD_LOGIC;
    LASER_CLK_3_P: in STD_LOGIC;
    LASER_CLK_3_N: in STD_LOGIC;
    STX: out STD_LOGIC;
    DDS_OUT: out STD_LOGIC;
    BEEPER_OUT: out STD_LOGIC
);
end Stitch;

architecture Behavioral of Stitch is


-- Declare Signals here
signal Stitch_CLK100 : STD_LOGIC;
signal Stitch_RX_IN : STD_LOGIC;
signal Stitch_LASER_CLK_1_P : STD_LOGIC;
signal Stitch_LASER_CLK_1_N : STD_LOGIC;
signal Stitch_LASER_SIGNAL_1_P : STD_LOGIC;
signal Stitch_LASER_SIGNAL_1_N : STD_LOGIC;
signal Stitch_LASER_CLK_2_P : STD_LOGIC;
signal Stitch_LASER_CLK_2_N : STD_LOGIC;
signal Stitch_LASER_SIGNAL_2_P : STD_LOGIC;
signal Stitch_LASER_SIGNAL_2_N : STD_LOGIC;
signal Stitch_LASER_CLK_3_P : STD_LOGIC;
signal Stitch_LASER_CLK_3_N : STD_LOGIC;
signal Stitch_LASER_SIGNAL_3_P : STD_LOGIC;
signal Stitch_LASER_SIGNAL_3_N : STD_LOGIC;
signal Stitch_BEEPER_OUT : STD_LOGIC;

signal iSerialReceiver_data : STD_LOGIC_VECTOR (7 downto 0);
signal iSerialReceiver_cmd_received : STD_LOGIC;

signal iCommandDecoder_readback_start : STD_LOGIC;
signal iCommandDecoder_sweeper_start : STD_LOGIC;
signal iCommandDecoder_n_ringup_mem_wr_en : STD_LOGIC;
signal iCommandDecoder_n_meas_mem_wr_en : STD_LOGIC;
signal iCommandDecoder_n_sweep_mem_wr_en : STD_LOGIC;
signal iCommandDecoder_dphi_mem_wr_en : STD_LOGIC;
signal iCommandDecoder_ddphi_mem_wr_en : STD_LOGIC;
signal iCommandDecoder_mem_addr : STD_LOGIC_VECTOR (9 downto 0);
signal iCommandDecoder_reg : STD_LOGIC_VECTOR (31 downto 0);
signal iCommandDecoder_n_freqs : STD_LOGIC_VECTOR (9 downto 0);

signal iSweeper_ddphi_mem_rd_addr : STD_LOGIC_VECTOR (9 downto 0);
signal iSweeper_dphi_mem_rd_addr : STD_LOGIC_VECTOR (9 downto 0);
signal iSweeper_n_ringup_mem_rd_addr : STD_LOGIC_VECTOR (9 downto 0);
signal iSweeper_n_meas_mem_rd_addr : STD_LOGIC_VECTOR (9 downto 0);
signal iSweeper_n_sweep_mem_rd_addr : STD_LOGIC_VECTOR (9 downto 0);
signal iSweeper_phi : STD_LOGIC_VECTOR (31 downto 0);
signal iSweeper_amp_mem_addr : STD_LOGIC_VECTOR (9 downto 0);
signal iSweeper_samples_mem_addr : STD_LOGIC_VECTOR (9 downto 0);
signal iSweeper_lockin_data_wr_en : STD_LOGIC;
signal iSweeper_lockin_en : STD_LOGIC;
signal iSweeper_lockin_clear : STD_LOGIC;
signal iSweeper_idle : STD_LOGIC;

signal iLaserReceiver_DISTANCE : STD_LOGIC_VECTOR (20 downto 0);
signal iLaserReceiver_DATA_READY : STD_LOGIC;
signal iLaserReceiver_RCP_START : STD_LOGIC;

signal iLockInAccumulator_dataCos : STD_LOGIC_VECTOR(63 downto 0);
signal iLockInAccumulator_dataSin : STD_LOGIC_VECTOR(63 downto 0);
signal iLockInAccumulator_n_samples : STD_LOGIC_VECTOR(31 downto 0);
signal iLockInAccumulator_idle : STD_LOGIC;

signal iDistanceUnwrapper_distance_out : STD_LOGIC_VECTOR(24 downto 0);
signal iDistanceUnwrapper_dr_out : STD_LOGIC;

signal iCordic_x_out : STD_LOGIC_VECTOR(20 DOWNTO 0);
signal iCordic_y_out : STD_LOGIC_VECTOR(20 DOWNTO 0);
signal iCordic_rdy : STD_LOGIC;

signal iReadbackControl_addr_rd : STD_LOGIC_VECTOR (9 downto 0);
signal iReadbackControl_idle : STD_LOGIC;
signal iReadbackControl_send : STD_LOGIC;

signal iSerialTransmitter_IDLE : STD_LOGIC;
signal iSerialTransmitter_TX : STD_LOGIC;

signal iMemSinCos_doutb : STD_LOGIC_VECTOR(127 DOWNTO 0);

signal iMemNsamp_doutb : STD_LOGIC_VECTOR(31 DOWNTO 0);

signal iMemNringup_doutb : STD_LOGIC_VECTOR(31 DOWNTO 0);

signal iMemNmeas_doutb : STD_LOGIC_VECTOR(31 DOWNTO 0);

signal iMemNsweep_doutb : STD_LOGIC_VECTOR(31 DOWNTO 0);

signal iMemDphi_doutb : STD_LOGIC_VECTOR(31 DOWNTO 0);

signal iMemDdphi_doutb : STD_LOGIC_VECTOR(31 DOWNTO 0);

signal iBeeper_Speakerout : STD_LOGIC;
signal iBeeper_Beeping : STD_LOGIC;


signal laserClock1: STD_LOGIC;
signal laserClock2: STD_LOGIC;
signal laserClock3: STD_LOGIC;
signal laserClock2N: STD_LOGIC;
signal laserData1: STD_LOGIC;
signal laserData2: STD_LOGIC;
signal laserData3: STD_LOGIC;
signal laserData1N: STD_LOGIC;

signal tx_data: STD_LOGIC_VECTOR(199 downto 0);

signal sincos_concat: STD_LOGIC_VECTOR(127 downto 0);

signal phi_clean: STD_LOGIC_VECTOR(31 downto 0);





-- Declare Compontents here
COMPONENT Sweeper
Port (  
    clk100: in STD_LOGIC;
    ddphi_mem_rd_data : in  STD_LOGIC_VECTOR (22 downto 0);
    dphi_mem_rd_data : in  STD_LOGIC_VECTOR (22 downto 0);
    n_ringup_mem_rd_data : in  STD_LOGIC_VECTOR (31 downto 0);
    n_meas_mem_rd_data : in  STD_LOGIC_VECTOR (31 downto 0);
    n_sweep_mem_rd_data : in  STD_LOGIC_VECTOR (31 downto 0);
    n_freqs : in  STD_LOGIC_VECTOR (9 downto 0);
    start : in  STD_LOGIC;
    lockin_idle : in  STD_LOGIC;
    ddphi_mem_rd_addr : out  STD_LOGIC_VECTOR (9 downto 0);
    dphi_mem_rd_addr : out  STD_LOGIC_VECTOR (9 downto 0);
    n_ringup_mem_rd_addr : out  STD_LOGIC_VECTOR (9 downto 0);
    n_meas_mem_rd_addr : out  STD_LOGIC_VECTOR (9 downto 0);
    n_sweep_mem_rd_addr : out  STD_LOGIC_VECTOR (9 downto 0);
    phi : out  STD_LOGIC_VECTOR (31 downto 0);
    amp_mem_addr : out  STD_LOGIC_VECTOR (9 downto 0);
    samples_mem_addr : out  STD_LOGIC_VECTOR (9 downto 0);
    lockin_data_wr_en : out  STD_LOGIC;
    lockin_en : out  STD_LOGIC;
    lockin_clear : out  STD_LOGIC;
    idle : out  STD_LOGIC
);
END COMPONENT;

COMPONENT LaserReceiver
Port ( 
    CLK100 : in  STD_LOGIC;
    SIGNAL_CLOCK : in  STD_LOGIC;
    DATA : in  STD_LOGIC;
    DISTANCE : out  STD_LOGIC_VECTOR (20 downto 0);
    DATA_READY : out  STD_LOGIC;
    RCP_START : out STD_LOGIC
);
END COMPONENT;

COMPONENT DistanceUnwrapper
Port (
    clk100 : in STD_LOGIC;
    en : in STD_LOGIC;
    dr : in STD_LOGIC;
    distance_in : in STD_LOGIC_VECTOR(20 downto 0);
    distance_out : out STD_LOGIC_VECTOR(24 downto 0);
    dr_out : out STD_LOGIC
);
END COMPONENT;

COMPONENT LockInAccumulator
Port ( 
    CLK100 : in  STD_LOGIC;
    sin : in STD_LOGIC_VECTOR(20 downto 0);
    cos : in STD_LOGIC_VECTOR(20 downto 0);
    data : in STD_LOGIC_VECTOR(20 downto 0);
    trig : in STD_LOGIC;
    clear : in STD_LOGIC;
    enable : in STD_LOGIC;
    dataCos : out STD_LOGIC_VECTOR(63 downto 0);
    dataSin : out STD_LOGIC_VECTOR(63 downto 0);
    n_samples : out STD_LOGIC_VECTOR(31 downto 0);
    idle : out STD_LOGIC
);
END COMPONENT;

COMPONENT SerialReceiver
Port ( 
    clk100			: in STD_LOGIC;
    s_data_in	 	: in  STD_LOGIC;
    data 			: out  STD_LOGIC_VECTOR (7 downto 0);
    cmd_received	: out  STD_LOGIC
);
END COMPONENT;

COMPONENT CommandDecoder
Port ( 
    clk100: in STD_LOGIC;
    data : in  STD_LOGIC_VECTOR (7 downto 0);
    cmd_received : in  STD_LOGIC;
    readback_idle : in  STD_LOGIC;
    sweeper_idle : in  STD_LOGIC;
    readback_start : out  STD_LOGIC;
    sweeper_start : out  STD_LOGIC;
    n_ringup_mem_wr_en : out  STD_LOGIC;
    n_meas_mem_wr_en : out  STD_LOGIC;
    n_sweep_mem_wr_en : out  STD_LOGIC;
    dphi_mem_wr_en : out  STD_LOGIC;
    ddphi_mem_wr_en : out  STD_LOGIC;
    mem_addr : out  STD_LOGIC_VECTOR (9 downto 0);
    reg : out  STD_LOGIC_VECTOR (31 downto 0);
    n_freqs : out  STD_LOGIC_VECTOR (9 downto 0)
);
END COMPONENT;

COMPONENT ReadbackControl
Port ( 
    clk100: in STD_LOGIC;
    start : in  STD_LOGIC;
    end_addr : in  STD_LOGIC_VECTOR (9 downto 0);
    serial_trans_idle : in  STD_LOGIC;
    addr_rd : out  STD_LOGIC_VECTOR (9 downto 0);
    idle : out  STD_LOGIC;
    send : out  STD_LOGIC
);
END COMPONENT;

COMPONENT SerialTransmitter
Port (
    CLK100 : in  STD_LOGIC;
    DR : in  STD_LOGIC;
    --Data : in  STD_LOGIC_VECTOR (7 downto 0);
    Data : in STD_LOGIC_VECTOR(199 downto 0);
    IDLE : out STD_LOGIC;
    TX : out  STD_LOGIC
);
END COMPONENT;

COMPONENT MemNsamp
PORT (
    clka : IN STD_LOGIC;
    wea : IN STD_LOGIC_VECTOR(0 DOWNTO 0);
    addra : IN STD_LOGIC_VECTOR(9 DOWNTO 0);
    dina : IN STD_LOGIC_VECTOR(31 DOWNTO 0);
    clkb : IN STD_LOGIC;
    addrb : IN STD_LOGIC_VECTOR(9 DOWNTO 0);
    doutb : OUT STD_LOGIC_VECTOR(31 DOWNTO 0)
);
END COMPONENT;

COMPONENT MemSinCos
PORT (
    clka : IN STD_LOGIC;
    wea : IN STD_LOGIC_VECTOR(0 DOWNTO 0);
    addra : IN STD_LOGIC_VECTOR(9 DOWNTO 0);
    dina : IN STD_LOGIC_VECTOR(127 DOWNTO 0);
    clkb : IN STD_LOGIC;
    addrb : IN STD_LOGIC_VECTOR(9 DOWNTO 0);
    doutb : OUT STD_LOGIC_VECTOR(127 DOWNTO 0)
);
END COMPONENT;

-- Used for MemNringup, MemNmeas, MemNsweep, MemDphi, MemDdphi
COMPONENT Mem32b1024
PORT (
    clka : IN STD_LOGIC;
    wea : IN STD_LOGIC_VECTOR(0 DOWNTO 0);
    addra : IN STD_LOGIC_VECTOR(9 DOWNTO 0);
    dina : IN STD_LOGIC_VECTOR(31 DOWNTO 0);
    clkb : IN STD_LOGIC;
    addrb : IN STD_LOGIC_VECTOR(9 DOWNTO 0);
    doutb : OUT STD_LOGIC_VECTOR(31 DOWNTO 0)
);
END COMPONENT;

COMPONENT cordic
PORT (
    phase_in : IN STD_LOGIC_VECTOR(31 DOWNTO 0);
    x_out : OUT STD_LOGIC_VECTOR(20 DOWNTO 0);
    y_out : OUT STD_LOGIC_VECTOR(20 DOWNTO 0);
    rdy : OUT STD_LOGIC;
    clk : IN STD_LOGIC;
    ce : IN STD_LOGIC
);
END COMPONENT;

COMPONENT Beeper
PORT (
    CLK100 : in  STD_LOGIC;
    Beep : in  STD_LOGIC;
    Speakerout : out  STD_LOGIC;
    Beeping: out STD_LOGIC
);
END COMPONENT;


begin
-- route signals here
OPT_OUT <= "0000000000";
Stitch_BEEPER_OUT <= iBeeper_Speakerout;
BEEPER_OUT <= Stitch_BEEPER_OUT;

Stitch_CLK100 <= CLK100;
Stitch_RX_IN <= NOT RX_IN;
Stitch_LASER_CLK_1_P <= LASER_CLK_1_P;
Stitch_LASER_CLK_1_N <= LASER_CLK_1_N;
Stitch_LASER_SIGNAL_1_P <= LASER_SIGNAL_1_P;
Stitch_LASER_SIGNAL_1_N <= LASER_SIGNAL_1_N;
Stitch_LASER_CLK_2_P <= LASER_CLK_2_P;
Stitch_LASER_CLK_2_N <= LASER_CLK_2_N;
Stitch_LASER_SIGNAL_2_P <= LASER_SIGNAL_2_P;
Stitch_LASER_SIGNAL_2_N <= LASER_SIGNAL_2_N;
Stitch_LASER_CLK_3_P <= LASER_CLK_3_P;
Stitch_LASER_CLK_3_N <= LASER_CLK_3_N;
Stitch_LASER_SIGNAL_3_P <= LASER_SIGNAL_3_P;
Stitch_LASER_SIGNAL_3_N <= LASER_SIGNAL_3_N;

laserClock2 <= NOT laserClock2N;
laserData1 <= NOT laserData1N;

--first startbit is done in the sender
tx_data(7 downto 0) <= iMemSinCos_doutb(7 downto 0);
tx_data(8) <= '1'; --stop bit
tx_data(9) <= '0'; --start bit
tx_data(17 downto 10) <= iMemSinCos_doutb(15 downto 8);
tx_data(18) <= '1'; --stop bit
tx_data(19) <= '0'; --start bit
tx_data(27 downto 20) <= iMemSinCos_doutb(23 downto 16);
tx_data(28) <= '1'; --stop bit
tx_data(29) <= '0'; --start bit
tx_data(37 downto 30) <= iMemSinCos_doutb(31 downto 24);
tx_data(38) <= '1'; --stop bit
tx_data(39) <= '0'; --start bit
tx_data(47 downto 40) <= iMemSinCos_doutb(39 downto 32);
tx_data(48) <= '1'; --stop bit
tx_data(49) <= '0'; --start bit
tx_data(57 downto 50) <= iMemSinCos_doutb(47 downto 40);
tx_data(58) <= '1'; --stop bit
tx_data(59) <= '0'; --start bit
tx_data(67 downto 60) <= iMemSinCos_doutb(55 downto 48);
tx_data(68) <= '1'; --stop bit
tx_data(69) <= '0'; --start bit
tx_data(77 downto 70) <= iMemSinCos_doutb(63 downto 56);
tx_data(78) <= '1'; --stop bit
tx_data(79) <= '0'; --start bit
tx_data(87 downto 80) <= iMemSinCos_doutb(71 downto 64);
tx_data(88) <= '1'; --stop bit
tx_data(89) <= '0'; --start bit
tx_data(97 downto 90) <= iMemSinCos_doutb(79 downto 72);
tx_data(98) <= '1'; --stop bit
tx_data(99) <= '0'; --start bit
tx_data(107 downto 100) <= iMemSinCos_doutb(87 downto 80);
tx_data(108) <= '1'; --stop bit
tx_data(109) <= '0'; --start bit
tx_data(117 downto 110) <= iMemSinCos_doutb(95 downto 88);
tx_data(118) <= '1'; --stop bit
tx_data(119) <= '0'; --start bit
tx_data(127 downto 120) <= iMemSinCos_doutb(103 downto 96);
tx_data(128) <= '1'; --stop bit
tx_data(129) <= '0'; --start bit
tx_data(137 downto 130) <= iMemSinCos_doutb(111 downto 104);
tx_data(138) <= '1'; --stop bit
tx_data(139) <= '0'; --start bit
tx_data(147 downto 140) <= iMemSinCos_doutb(119 downto 112);
tx_data(148) <= '1'; --stop bit
tx_data(149) <= '0'; --start bit
tx_data(157 downto 150) <= iMemSinCos_doutb(127 downto 120);
tx_data(158) <= '1'; --stop bit
tx_data(159) <= '0'; --start bit
tx_data(167 downto 160) <= iMemNsamp_doutb(7 downto 0);
tx_data(168) <= '1'; --stop bit
tx_data(169) <= '0'; --start bit
tx_data(177 downto 170) <= iMemNsamp_doutb(15 downto 8);
tx_data(178) <= '1'; --stop bit
tx_data(179) <= '0'; --start bit
tx_data(187 downto 180) <= iMemNsamp_doutb(23 downto 16);
tx_data(188) <= '1'; --stop bit
tx_data(189) <= '0'; --start bit
tx_data(197 downto 190) <= iMemNsamp_doutb(31 downto 24);
tx_data(198) <= '1'; --stop bit
tx_data(199) <= '1'; --additional stop bit

sincos_concat(63 downto 0) <= iLockInAccumulator_dataCos;
sincos_concat(127 downto 64) <= iLockInAccumulator_dataSin;

STX <= iSerialTransmitter_TX;

DDS_OUT <= iSweeper_phi(28);

LEDS(0) <= '1';
LEDS(1) <= NOT iSweeper_idle;
LEDS(2) <= NOT iReadbackControl_idle;

phi_clean(31 downto 1) <= iSweeper_phi(30 downto 0);
phi_clean(0) <= '0';

-- instantiate components here

--BUFS for LVDS in
i_IBUFDS1 : IBUFDS generic map(IOSTANDARD => "DEFAULT", DIFF_TERM => TRUE) port map(laserClock1, Stitch_LASER_CLK_1_P, Stitch_LASER_CLK_1_N);
i_IBUFDS2 : IBUFDS generic map(IOSTANDARD => "DEFAULT", DIFF_TERM => TRUE) port map(laserClock2N, Stitch_LASER_CLK_2_P, Stitch_LASER_CLK_2_N);
i_IBUFDS3 : IBUFDS generic map(IOSTANDARD => "DEFAULT", DIFF_TERM => TRUE) port map(laserClock3, Stitch_LASER_CLK_3_P, Stitch_LASER_CLK_3_N);
i_IBUFDS4 : IBUFDS generic map(IOSTANDARD => "DEFAULT", DIFF_TERM => TRUE) port map(laserData1N, Stitch_LASER_SIGNAL_1_P, Stitch_LASER_SIGNAL_1_N);
i_IBUFDS5 : IBUFDS generic map(IOSTANDARD => "DEFAULT", DIFF_TERM => TRUE) port map(laserData2, Stitch_LASER_SIGNAL_2_P, Stitch_LASER_SIGNAL_2_N);
i_IBUFDS6 : IBUFDS generic map(IOSTANDARD => "DEFAULT", DIFF_TERM => TRUE) port map(laserData3, Stitch_LASER_SIGNAL_3_P, Stitch_LASER_SIGNAL_3_N);



iSerialReceiver : SerialReceiver port map(
	clk100 => Stitch_CLK100,  -- in STD_LOGIC
	s_data_in => Stitch_RX_IN,  -- in  STD_LOGIC
	data => iSerialReceiver_data,  -- out  STD_LOGIC_VECTOR (7 downto 0)
	cmd_received => iSerialReceiver_cmd_received  -- out  STD_LOGIC
);

iCommandDecoder : CommandDecoder port map(
	clk100 => Stitch_CLK100,  -- in STD_LOGIC
	data => iSerialReceiver_data,  -- in  STD_LOGIC_VECTOR (7 downto 0)
	cmd_received => iSerialReceiver_cmd_received,  -- in  STD_LOGIC
	readback_idle => iReadbackControl_idle,  -- in  STD_LOGIC
	sweeper_idle => iSweeper_idle,  -- in  STD_LOGIC
	readback_start => iCommandDecoder_readback_start,  -- out  STD_LOGIC
	sweeper_start => iCommandDecoder_sweeper_start,  -- out  STD_LOGIC
	n_ringup_mem_wr_en => iCommandDecoder_n_ringup_mem_wr_en,  -- out  STD_LOGIC
	n_meas_mem_wr_en => iCommandDecoder_n_meas_mem_wr_en,  -- out  STD_LOGIC
	n_sweep_mem_wr_en => iCommandDecoder_n_sweep_mem_wr_en,  -- out  STD_LOGIC
	dphi_mem_wr_en => iCommandDecoder_dphi_mem_wr_en,  -- out  STD_LOGIC
	ddphi_mem_wr_en => iCommandDecoder_ddphi_mem_wr_en,  -- out  STD_LOGIC
	mem_addr => iCommandDecoder_mem_addr,  -- out  STD_LOGIC_VECTOR (9 downto 0)
	reg => iCommandDecoder_reg,  -- out  STD_LOGIC_VECTOR (31 downto 0)
	n_freqs => iCommandDecoder_n_freqs  -- out  STD_LOGIC_VECTOR (9 downto 0)
);

iSweeper : Sweeper port map(
	clk100 => Stitch_CLK100,  -- in STD_LOGIC
	ddphi_mem_rd_data => iMemDdphi_doutb(22 downto 0),  -- in  STD_LOGIC_VECTOR (22 downto 0)
	dphi_mem_rd_data => iMemDphi_doutb(22 downto 0),  -- in  STD_LOGIC_VECTOR (22 downto 0)
	n_ringup_mem_rd_data => iMemNringup_doutb,  -- in  STD_LOGIC_VECTOR (31 downto 0)
	n_meas_mem_rd_data => iMemNmeas_doutb,  -- in  STD_LOGIC_VECTOR (31 downto 0)
	n_sweep_mem_rd_data => iMemNsweep_doutb,  -- in  STD_LOGIC_VECTOR (31 downto 0)
	n_freqs => iCommandDecoder_n_freqs,  -- in  STD_LOGIC_VECTOR (9 downto 0)
	start => iCommandDecoder_sweeper_start,  -- in  STD_LOGIC
	lockin_idle => iLockInAccumulator_idle,  -- in  STD_LOGIC
	ddphi_mem_rd_addr => iSweeper_ddphi_mem_rd_addr,  -- out  STD_LOGIC_VECTOR (9 downto 0)
	dphi_mem_rd_addr => iSweeper_dphi_mem_rd_addr,  -- out  STD_LOGIC_VECTOR (9 downto 0)
	n_ringup_mem_rd_addr => iSweeper_n_ringup_mem_rd_addr,  -- out  STD_LOGIC_VECTOR (9 downto 0)
	n_meas_mem_rd_addr => iSweeper_n_meas_mem_rd_addr,  -- out  STD_LOGIC_VECTOR (9 downto 0)
	n_sweep_mem_rd_addr => iSweeper_n_sweep_mem_rd_addr,  -- out  STD_LOGIC_VECTOR (9 downto 0)
	phi => iSweeper_phi,  -- out  STD_LOGIC_VECTOR (31 downto 0)
	amp_mem_addr => iSweeper_amp_mem_addr,  -- out  STD_LOGIC_VECTOR (9 downto 0)
	samples_mem_addr => iSweeper_samples_mem_addr,  -- out  STD_LOGIC_VECTOR (9 downto 0)
	lockin_data_wr_en => iSweeper_lockin_data_wr_en,  -- out  STD_LOGIC
	lockin_en => iSweeper_lockin_en,  -- out  STD_LOGIC
	lockin_clear => iSweeper_lockin_clear,  -- out  STD_LOGIC
	idle => iSweeper_idle  -- out  STD_LOGIC
);

iLaserReceiver : LaserReceiver port map(
	CLK100 => Stitch_CLK100,  -- in  STD_LOGIC
	SIGNAL_CLOCK => laserClock3,  -- in  STD_LOGIC
	DATA => laserData3,  -- in  STD_LOGIC
	DISTANCE => iLaserReceiver_DISTANCE,  -- out  STD_LOGIC_VECTOR (20 downto 0)
	DATA_READY => iLaserReceiver_DATA_READY,  -- out  STD_LOGIC
    RCP_START => iLaserReceiver_RCP_START  --out STD_LOGIC
);

iLockInAccumulator : LockInAccumulator port map(
	CLK100 => Stitch_CLK100,  -- in  STD_LOGIC
	sin => iCordic_y_out,  -- in STD_LOGIC_VECTOR(20 downto 0)
	cos => iCordic_x_out,  -- in STD_LOGIC_VECTOR(20 downto 0)
	data => iDistanceUnwrapper_distance_out,  -- in STD_LOGIC_VECTOR(20 downto 0)
	trig => iDistanceUnwrapper_dr_out,  -- in STD_LOGIC
	clear => iSweeper_lockin_clear,  -- in STD_LOGIC
	enable => iSweeper_lockin_en,  -- in STD_LOGIC
	dataCos => iLockInAccumulator_dataCos,  -- out STD_LOGIC_VECTOR(63 downto 0)
	dataSin => iLockInAccumulator_dataSin,  -- out STD_LOGIC_VECTOR(63 downto 0)
	n_samples => iLockInAccumulator_n_samples,  -- out STD_LOGIC_VECTOR(31 downto 0)
	idle => iLockInAccumulator_idle  -- out STD_LOGIC
);

iDistanceUnwrapper : DistanceUnwrapper port map(
    clk100 => Stitch_CLK100,
    en => iSweeper_lockin_en,
    dr => iLaserReceiver_DATA_READY,
    distance_in => iLaserReceiver_DISTANCE,
    distance_out => iDistanceUnwrapper_distance_out,
    dr_out => iDistanceUnwrapper_dr_out
);

--TODO: The clocking and data input and stuff may need some optimizations. Refer to the manual in ISE, esp. pertaining parallel vs word serial
iCordic : cordic port map(
	phase_in => phi_clean,  -- IN STD_LOGIC_VECTOR(31 DOWNTO 0)
	x_out => iCordic_x_out,  -- OUT STD_LOGIC_VECTOR(20 DOWNTO 0)
	y_out => iCordic_y_out,  -- OUT STD_LOGIC_VECTOR(20 DOWNTO 0)
	rdy => iCordic_rdy,  -- OUT STD_LOGIC
	clk => Stitch_CLK100,  -- IN STD_LOGIC
	ce => '1'  -- IN STD_LOGIC
);

iReadbackControl : ReadbackControl port map(
	clk100 => Stitch_CLK100,  -- in STD_LOGIC
	start => iCommandDecoder_readback_start,  -- in  STD_LOGIC
	end_addr => iCommandDecoder_n_freqs,  -- in  STD_LOGIC_VECTOR (9 downto 0)
	serial_trans_idle => iSerialTransmitter_IDLE,  -- in  STD_LOGIC
	addr_rd => iReadbackControl_addr_rd,  -- out  STD_LOGIC_VECTOR (9 downto 0)
	idle => iReadbackControl_idle,  -- out  STD_LOGIC
	send => iReadbackControl_send  -- out  STD_LOGIC
);

iSerialTransmitter : SerialTransmitter port map(
	CLK100 => Stitch_CLK100,  -- in  STD_LOGIC
	DR => iReadbackControl_send,  -- in  STD_LOGIC
	--Data => ,  -- in  STD_LOGIC_VECTOR (7 downto 0)
	Data => tx_data,  -- in STD_LOGIC_VECTOR(199 downto 0)
	IDLE => iSerialTransmitter_IDLE,  -- out STD_LOGIC
	TX => iSerialTransmitter_TX  -- out  STD_LOGIC
);

iMemSinCos : MemSinCos port map(
	clka => Stitch_CLK100,  -- IN STD_LOGIC
	wea(0) => iSweeper_lockin_data_wr_en,  -- IN STD_LOGIC_VECTOR(0 DOWNTO 0)
	addra => iSweeper_amp_mem_addr,  -- IN STD_LOGIC_VECTOR(9 DOWNTO 0)
	dina => sincos_concat,  -- IN STD_LOGIC_VECTOR(127 DOWNTO 0)
	clkb => Stitch_CLK100,  -- IN STD_LOGIC
	addrb => iReadbackControl_addr_rd,  -- IN STD_LOGIC_VECTOR(9 DOWNTO 0)
	doutb => iMemSinCos_doutb  -- OUT STD_LOGIC_VECTOR(127 DOWNTO 0)
);

iMemNsamp : MemNsamp port map(
	clka => Stitch_CLK100,  -- IN STD_LOGIC
	wea(0) => iSweeper_lockin_data_wr_en,  -- IN STD_LOGIC_VECTOR(0 DOWNTO 0)
	addra => iSweeper_samples_mem_addr,  -- IN STD_LOGIC_VECTOR(9 DOWNTO 0)
	dina => iLockInAccumulator_n_samples,  -- IN STD_LOGIC_VECTOR(31 DOWNTO 0)
	clkb => Stitch_CLK100,  -- IN STD_LOGIC
	addrb => iReadbackControl_addr_rd,  -- IN STD_LOGIC_VECTOR(9 DOWNTO 0)
	doutb => iMemNsamp_doutb  -- OUT STD_LOGIC_VECTOR(31 DOWNTO 0)
);

iMemNringup : Mem32b1024 port map(
	clka => Stitch_CLK100,  -- IN STD_LOGIC
	wea(0) => iCommandDecoder_n_ringup_mem_wr_en,  -- IN STD_LOGIC_VECTOR(0 DOWNTO 0)
	addra => iCommandDecoder_mem_addr,  -- IN STD_LOGIC_VECTOR(9 DOWNTO 0)
	dina => iCommandDecoder_reg,  -- IN STD_LOGIC_VECTOR(31 DOWNTO 0)
	clkb => Stitch_CLK100,  -- IN STD_LOGIC
	addrb => iSweeper_n_ringup_mem_rd_addr,  -- IN STD_LOGIC_VECTOR(9 DOWNTO 0)
	doutb => iMemNringup_doutb  -- OUT STD_LOGIC_VECTOR(31 DOWNTO 0)
);

iMemNmeas : Mem32b1024 port map(
	clka => Stitch_CLK100,  -- IN STD_LOGIC
	wea(0) => iCommandDecoder_n_meas_mem_wr_en,  -- IN STD_LOGIC_VECTOR(0 DOWNTO 0)
	addra => iCommandDecoder_mem_addr,  -- IN STD_LOGIC_VECTOR(9 DOWNTO 0)
	dina => iCommandDecoder_reg,  -- IN STD_LOGIC_VECTOR(31 DOWNTO 0)
	clkb => Stitch_CLK100,  -- IN STD_LOGIC
	addrb => iSweeper_n_meas_mem_rd_addr,  -- IN STD_LOGIC_VECTOR(9 DOWNTO 0)
	doutb => iMemNmeas_doutb  -- OUT STD_LOGIC_VECTOR(31 DOWNTO 0)
);

iMemNsweep : Mem32b1024 port map(
	clka => Stitch_CLK100,  -- IN STD_LOGIC
	wea(0) => iCommandDecoder_n_sweep_mem_wr_en,  -- IN STD_LOGIC_VECTOR(0 DOWNTO 0)
	addra => iCommandDecoder_mem_addr,  -- IN STD_LOGIC_VECTOR(9 DOWNTO 0)
	dina => iCommandDecoder_reg,  -- IN STD_LOGIC_VECTOR(31 DOWNTO 0)
	clkb => Stitch_CLK100,  -- IN STD_LOGIC
	addrb => iSweeper_n_sweep_mem_rd_addr,  -- IN STD_LOGIC_VECTOR(9 DOWNTO 0)
	doutb => iMemNsweep_doutb  -- OUT STD_LOGIC_VECTOR(31 DOWNTO 0)
);

iMemDphi : Mem32b1024 port map(
	clka => Stitch_CLK100,  -- IN STD_LOGIC
	wea(0) => iCommandDecoder_dphi_mem_wr_en,  -- IN STD_LOGIC_VECTOR(0 DOWNTO 0)
	addra => iCommandDecoder_mem_addr,  -- IN STD_LOGIC_VECTOR(9 DOWNTO 0)
	dina => iCommandDecoder_reg,  -- IN STD_LOGIC_VECTOR(31 DOWNTO 0)
	clkb => Stitch_CLK100,  -- IN STD_LOGIC
	addrb => iSweeper_dphi_mem_rd_addr,  -- IN STD_LOGIC_VECTOR(9 DOWNTO 0)
	doutb => iMemDphi_doutb  -- OUT STD_LOGIC_VECTOR(31 DOWNTO 0)
);

iMemDdphi : Mem32b1024 port map(
	clka => Stitch_CLK100,  -- IN STD_LOGIC
	wea(0) => iCommandDecoder_ddphi_mem_wr_en,  -- IN STD_LOGIC_VECTOR(0 DOWNTO 0)
	addra => iCommandDecoder_mem_addr,  -- IN STD_LOGIC_VECTOR(9 DOWNTO 0)
	dina => iCommandDecoder_reg,  -- IN STD_LOGIC_VECTOR(31 DOWNTO 0)
	clkb => Stitch_CLK100,  -- IN STD_LOGIC
	addrb => iSweeper_ddphi_mem_rd_addr,  -- IN STD_LOGIC_VECTOR(9 DOWNTO 0)
	doutb => iMemDdphi_doutb  -- OUT STD_LOGIC_VECTOR(31 DOWNTO 0)
);

iBeeper : Beeper port map(
    CLK100 => Stitch_CLK100,
    Beep => iSerialReceiver_cmd_received,
    Speakerout => iBeeper_Speakerout,
    Beeping => iBeeper_Beeping
);

-- processes here, probably have none

end Behavioral;

